<?php
namespace Neklo\Scheduler\Controller\Adminhtml\Job;

use Neklo\Scheduler\Controller\Adminhtml;
use Neklo\Scheduler\Api\Data\JobInterface;

class Schedule extends Adminhtml\Action
{
    /**
     * Redirect url
     */
    const REDIRECT_URL = '*/index/jobs';

    /**
     *
     * @var \Neklo\Scheduler\Model\ScheduleFactory
     */
    protected $scheduleFactory;

    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        \Magento\Framework\View\Result\PageFactory $resultPageFactory,
        \Neklo\Scheduler\Model\ScheduleFactory $scheduleFactory,
        \Magento\Framework\Registry $registry
    ) {
        parent::__construct($context, $resultPageFactory, $registry);
        $this->scheduleFactory = $scheduleFactory;
    }

    public function execute()
    {
        $id = $this->getRequest()->getParam(JobInterface::JOB_ID);
        $model = $this->_objectManager->create('Neklo\Scheduler\Model\Job');
        $errMsg = [];
        try {
            if ($id) {
                $model->load($id);
                if (!$model->getId()) {
                    $errMsg[] = __('This job ID(%1) no longer exists.', $id);
                } else {
                    $this->scheduleIt($model);
                }
            }
        } catch (\Exception $e) {
            $errMsg[] = __($e->getMessage());
        }

        if (!empty($errMsg)) {
            $this->messageManager
                ->addError(implode('<br>', $errMsg));
        }
        $resultRedirect = $this->resultRedirectFactory->create();
        return $resultRedirect->setPath(self::REDIRECT_URL);
    }

    /**
     *
     * @param \Neklo\Scheduler\Model\Job $job
     */
    protected function scheduleIt(\Neklo\Scheduler\Model\Job $job)
    {
        $schedule = $this->scheduleFactory->create()
            ->initializeFromJob($job)
            ->setJob($job)
            ->setScheduledReason("Scheduled From Admin")
            ->schedule()
            ->save();

        $this->messageManager
            ->addSuccess(__('Job "%1" has been scheduled.', $job->getName()));
    }
}
